@@ -1,9 +1,10 @@
+import { DataFrameView, KeyValue, MutableDataFrame } from '@grafana/data';
 import { ElasticResponse } from '../elastic_response';
 
 describe('ElasticResponse', () => {
   let targets;
-  let response;
-  let result;
+  let response: any;
+  let result: any;
 
   describe('simple query and count', () => {
     beforeEach(() => {
@@ -48,7 +49,7 @@ describe('ElasticResponse', () => {
   });
 
   describe('simple query count & avg aggregation', () => {
-    let result;
+    let result: any;
 
     beforeEach(() => {
       targets = [
@@ -97,7 +98,7 @@ describe('ElasticResponse', () => {
   });
 
   describe('single group by query one metric', () => {
-    let result;
+    let result: any;
 
     beforeEach(() => {
       targets = [
@@ -149,7 +150,7 @@ describe('ElasticResponse', () => {
   });
 
   describe('single group by query two metrics', () => {
-    let result;
+    let result: any;
 
     beforeEach(() => {
       targets = [
@@ -209,7 +210,7 @@ describe('ElasticResponse', () => {
   });
 
   describe('with percentiles ', () => {
-    let result;
+    let result: any;
 
     beforeEach(() => {
       targets = [
@@ -257,7 +258,7 @@ describe('ElasticResponse', () => {
   });
 
   describe('with extended_stats', () => {
-    let result;
+    let result: any;
 
     beforeEach(() => {
       targets = [
@@ -333,7 +334,7 @@ describe('ElasticResponse', () => {
   });
 
   describe('single group by with alias pattern', () => {
-    let result;
+    let result: any;
 
     beforeEach(() => {
       targets = [
@@ -394,7 +395,7 @@ describe('ElasticResponse', () => {
   });
 
   describe('histogram response', () => {
-    let result;
+    let result: any;
 
     beforeEach(() => {
       targets = [
@@ -426,7 +427,7 @@ describe('ElasticResponse', () => {
   });
 
   describe('with two filters agg', () => {
-    let result;
+    let result: any;
 
     beforeEach(() => {
       targets = [
@@ -582,6 +583,59 @@ describe('ElasticResponse', () => {
     });
   });
 
+  describe('No group by time with percentiles ', () => {
+    let result: any;
+
+    beforeEach(() => {
+      targets = [
+        {
+          refId: 'A',
+          metrics: [{ type: 'percentiles', field: 'value', settings: { percents: [75, 90] }, id: '1' }],
+          bucketAggs: [{ type: 'term', field: 'id', id: '3' }],
+        },
+      ];
+      response = {
+        responses: [
+          {
+            aggregations: {
+              '3': {
+                buckets: [
+                  {
+                    '1': { values: { '75': 3.3, '90': 5.5 } },
+                    doc_count: 10,
+                    key: 'id1',
+                  },
+                  {
+                    '1': { values: { '75': 2.3, '90': 4.5 } },
+                    doc_count: 15,
+                    key: 'id2',
+                  },
+                ],
+              },
+            },
+          },
+        ],
+      };
+
+      result = new ElasticResponse(targets, response).getTimeSeries();
+    });
+
+    it('should return table', () => {
+      expect(result.data.length).toBe(1);
+      expect(result.data[0].type).toBe('table');
+      expect(result.data[0].columns[0].text).toBe('id');
+      expect(result.data[0].columns[1].text).toBe('p75 value');
+      expect(result.data[0].columns[2].text).toBe('p90 value');
+      expect(result.data[0].rows.length).toBe(2);
+      expect(result.data[0].rows[0][0]).toBe('id1');
+      expect(result.data[0].rows[0][1]).toBe(3.3);
+      expect(result.data[0].rows[0][2]).toBe(5.5);
+      expect(result.data[0].rows[1][0]).toBe('id2');
+      expect(result.data[0].rows[1][1]).toBe(2.3);
+      expect(result.data[0].rows[1][2]).toBe(4.5);
+    });
+  });
+
   describe('Multiple metrics of same type', () => {
     beforeEach(() => {
       targets = [
@@ -667,7 +721,7 @@ describe('ElasticResponse', () => {
   });
 
   describe('with bucket_script ', () => {
-    let result;
+    let result: any;
 
     beforeEach(() => {
       targets = [
@@ -731,4 +785,113 @@ describe('ElasticResponse', () => {
       expect(result.data[2].datapoints[1][0]).toBe(12);
     });
   });
+
+  describe('simple logs query and count', () => {
+    beforeEach(() => {
+      targets = [
+        {
+          refId: 'A',
+          metrics: [{ type: 'count', id: '1' }],
+          bucketAggs: [{ type: 'date_histogram', settings: { interval: 'auto' }, id: '2' }],
+          context: 'explore',
+          interval: '10s',
+          isLogsQuery: true,
+          key: 'Q-1561369883389-0.7611823271062786-0',
+          liveStreaming: false,
+          maxDataPoints: 1620,
+          query: '',
+          timeField: '@timestamp',
+        },
+      ];
+      response = {
+        responses: [
+          {
+            aggregations: {
+              '2': {
+                buckets: [
+                  {
+                    doc_count: 10,
+                    key: 1000,
+                  },
+                  {
+                    doc_count: 15,
+                    key: 2000,
+                  },
+                ],
+              },
+            },
+            hits: {
+              hits: [
+                {
+                  _id: 'fdsfs',
+                  _type: '_doc',
+                  _index: 'mock-index',
+                  _source: {
+                    '@timestamp': '2019-06-24T09:51:19.765Z',
+                    host: 'djisaodjsoad',
+                    message: 'hello, i am a message',
+                  },
+                  fields: {
+                    '@timestamp': ['2019-06-24T09:51:19.765Z'],
+                  },
+                },
+                {
+                  _id: 'kdospaidopa',
+                  _type: '_doc',
+                  _index: 'mock-index',
+                  _source: {
+                    '@timestamp': '2019-06-24T09:52:19.765Z',
+                    host: 'dsalkdakdop',
+                    message: 'hello, i am also message',
+                  },
+                  fields: {
+                    '@timestamp': ['2019-06-24T09:52:19.765Z'],
+                  },
+                },
+              ],
+            },
+          },
+        ],
+      };
+
+      result = new ElasticResponse(targets, response).getLogs();
+    });
+
+    it('should return histogram aggregation and documents', () => {
+      expect(result.data.length).toBe(2);
+      const logResults = result.data[0] as MutableDataFrame;
+      const fields = logResults.fields.map(f => {
+        return {
+          name: f.name,
+          type: f.type,
+        };
+      });
+
+      expect(fields).toContainEqual({ name: '@timestamp', type: 'time' });
+      expect(fields).toContainEqual({ name: 'host', type: 'string' });
+      expect(fields).toContainEqual({ name: 'message', type: 'string' });
+
+      let rows = new DataFrameView(logResults);
+      for (let i = 0; i < rows.length; i++) {
+        const r = rows.get(i);
+        expect(r._id).toEqual(response.responses[0].hits.hits[i]._id);
+        expect(r._type).toEqual(response.responses[0].hits.hits[i]._type);
+        expect(r._index).toEqual(response.responses[0].hits.hits[i]._index);
+        expect(r._source).toEqual(response.responses[0].hits.hits[i]._source);
+      }
+
+      // Make a map from the histogram results
+      const hist: KeyValue<number> = {};
+      const histogramResults = new MutableDataFrame(result.data[1]);
+      rows = new DataFrameView(histogramResults);
+      for (let i = 0; i < rows.length; i++) {
+        const row = rows.get(i);
+        hist[row.Time] = row.Count;
+      }
+
+      response.responses[0].aggregations['2'].buckets.forEach((bucket: any) => {
+        expect(hist[bucket.key]).toEqual(bucket.doc_count);
+      });
+    });
+  });
 });
